indexing

   title:       "SCIENTIFIC_FORMAT usage example";
   cluster:     "format/science/examples/example [format/science/examples/example]";
   project:     "The Universal Simple Formatting Library (USFL)";
   copyright:   "SIG Computer, 1995";
   author:      "Andy Malakov  (andy@eiffel.ru) and";
   original:    08,Sep,1995;
   version:     1.0;
   last_change:
   done_at:     "SIG Computer (info@eiffel.ru)";
   extrnl_name: "example.e"

class EXAMPLE
      -- Class illustrate SCIENTIFIC_FORMAT usage.

   inherit
      SCIENTIFIC_FORMAT
         rename make as sformat_make
         end

   creation
     make

   feature {ANY}

      make is

      local
         i, width, precision, count : INTEGER
         user_name, some_str : STRING
         a : ARRAY [ANY]
         datetime : TIMESTAMP
      do
         sformat_make
         !!a.make(1,3)
         from i := 1 until i > a.count loop a.put(i,i) i:=i+1 end

   ------------------------------------------------------------
   -- You may mix-up several types in format string
   -- argument must conform to corresponding typechar
   ------------------------------------------------------------
      printf ( "!i !x !o !u !c !b !s !e !f %N",
             <<
                -123,
                256,
                256,
                123,
                'c',
                true,
                "some string",
                3.1415934563,
                3.1415934563
             >>
             )

   ------------------------------------------------------------
   -- Before typechar you may specify output width and precision
   --
   -- SYNTAX: ![width][.precision]typechar
   --
   -- width - Minimum number of characters to print, padding
   --         with blanks or zeros
   --
   -- precision - Maximum number of characters to print; for
   --             integers, minimum number of digits to print
   --
   ------------------------------------------------------------
      printf ( "!5i !4x !6o !u !c !b !3s !8e !7.8f %N",
             <<
                -123,
                256,
                256,
                123,
                'c',
                true,
                "some string",
                3.1415934563,
                3.1415934563
             >>
             )

   -----------------------------------------------------------------------
   -- You may specify following format flags:
   -- =======================================
   -- '-' Means left alignment (by default value aligns to rigth)
   --
   -- '^' Means center alignment (by default value aligns to rigth)
   --
   -- '+' Prefix  the  output value with a sign (+ or -) if the output
   --     value is of a signed type. (by default only '-' printed)
   --
   -- ' ' Similar to '+' but add space chacacter instead of plus.
   --
   -- '0' If width is prefixed with 0, zeros are added until the
   --     minimum width is reached (by default added spaces)
   --
   -- Restrictions:
   --    (' ' after '+') and ('0' with '-','^') combinations are incorrect
   -----------------------------------------------------------------------
      printf ( "!05i !04x !6o !u !c !b !-.3s !00.5e !+7.8f %N",
             <<
                -123,
                256,
                256,
                123,
                'c',
                true,
                "some string",
                3.1415934563,
                3.1415934563
             >>
             )

      printf ( "(!^10s) (!^10i) (!^10e) %N", <<"abc", 123, 1.23>> )

   ------------------------------------------------------------
   -- Of course you may use not only manifest arrays
   -- Be sure that any array element is non Void
   ------------------------------------------------------------
      printf ( "Arr3 (!02i) (!02i) (!02i) %N", a )

   ------------------------------------------------------------
   -- Class SCIENTIFIC_FORMAT also provide short form printing methods
   -- they print single value using default format string
   ------------------------------------------------------------

      set_default_format ( "!-10d"  )
      set_default_format ( "-10.3f" ) -- escape character is optional here

      print ( 256 )
      print ( 123.456 )
      print ( "What is the next?%N" )

      -- You may change default format
      set_default_format ( "!012x"  )
      set_default_format ( "-10.3e" ) -- escape character is optional here
      set_default_format ( "-20.10s" )

      print ( 123 )
      print ( 123.456 )
      print ( "What is the next?%N" )

   ------------------------------------------------------------
   -- Class SCIENTIFIC_FORMAT also provide methods to convert
   -- arguments array or single value to string object
   ------------------------------------------------------------
      some_str := sprintf ( "%N!s's salary is $!i", << "John", 1000 >> )
      some_str.append ( to_string ( " Mary spend $!i :-)%N", 1001  ))

      set_default_format ( "!s" ) -- reset default format
      print ( some_str )

   ------------------------------------------------------------
   -- You may take width or precision (or both) from arguments
   -- To do this put asterisk instead of format field
   -- In such case format arguments must conform to INTEGER
   ------------------------------------------------------------
      width:=10 precision:=5 count:=123
      printf ( "Custom format: !*.*i%N", << width, precision, count >>)
      printf ( "Custom format: !*i%N", << width, count >>)
      printf ( "Custom format: !.*i%N", << precision, count >>)

   ------------------------------------------------------------
   -- To output character same as default escape write it twice
   ------------------------------------------------------------
      user_name := "Andy"
      printf ( "Hello !s !!", <<user_name>> )

      -- Or temporary change value of default_secape
      set_escape ('$')
      printf ( "Hello $s !", <<user_name>> )
      set_escape ('!')

   ------------------------------------------------------------
   -- Arguments array may be Void in such cases format string
   -- must contain no escape sequences
   ------------------------------------------------------------
      printf ( "Hello !!%N", Void )

      !!datetime.set_to_now

      printf ( "Today is !a now !t %N", << datetime, datetime >> )
      printf ( " !8.0a !8.12t %N", << datetime, datetime >> )
      printf ( " 0-!.0a 1-!.1a 2-!.2a 3-!.3a 4-!.4a 5-!.5a 6-!.6a 7-!.7a 8-!.8a %N",
               << datetime, datetime, datetime, datetime, datetime,
                  datetime, datetime, datetime, datetime  >> )

      end -- make

end -- class EXAMPLE

