-- e and f should be no problem for any qualified computer scientist - but
-- they have to write it ... so its better if they do it together.
--                                              F. Monninger

indexing

   title:       "Fixed point formatter";
   cluster:     "format/science [format/science]";
   project:     "The Universal Simple Formatting Library (USFL)";
   copyright:   "SIG Computer, 1995";
   author:      "Igor Nesterof (nest@eiffel.ru)";
   original:    08,Sep,1995;
   version:     1.0;
   done_at:     "SIG Computer (info@eiffel.ru)";
   extrnl_name: "fix_fmt.e"

class FIXED_POINT_FORMATTER

inherit
   ABSTRACT_JUSTIFY_FORMATTER
      redefine
         make, set_sign, set_align_char, output
   end
   EXTENDED_MATH

creation
   make

feature {ABSTRACT_FORMATTER}

   -- number image saves as
   -- number := fsign*(int_part+fract_part)
   fsign : INTEGER
   int_part : INTEGER
   fract_part : REAL
   zero_strip : BOOLEAN

   -- formatter for integer part. formatted on-fly
   i_f : INTEGER_FORMATTER -- for integer part formatting

feature

------------------------------------------------------------make
   -- make split number on parts to simplify formatting
   make,frozen fixed_point_make(f : REAL_REF) is
   require else
      format_real : f /= Void
   local
      fa : REAL
   do
      fa := abs(f.item)
      fsign := real_sign(f.item)
      int_part := floor(fa)
      fract_part := fa - int_part
      !!i_f.make(int_part*fsign)
      abstract_justify_make(fa)
      set_precision(6)
      zero_strip := FALSE
   end
------------------------------------------------------------set_sign
   set_sign(s:CHARACTER) is
   do
      i_f.set_sign(s)
   end
------------------------------------------------------------set_align_char
   set_align_char(c:CHARACTER) is
   do
      i_f.set_align_char(c)
   end
------------------------------------------------------------ set_zero_strip
   set_zero_strip(zs : BOOLEAN) is
   do
      zero_strip := zs
   end

------------------------------------------------------------output
   output, frozen fixed_output : STRING is
   local
      f_f : UNSIGNED_DECIMAL_INTEGER_FORMATTER
   do
      Result := make_fract_part
      -- make integer part and finish alignment
      inspect alignment
      when align_left then
         i_f.set_width(1)
         Result.prepend(i_f.output)
         Result := right_pad(Result,' ',width)
      when align_right then
         i_f.set_width(width-Result.count)
         i_f.set_align_right
         Result.prepend(i_f.output)
      when align_center then
         i_f.set_width(1)
         Result.prepend(i_f.output)
         Result := center_pad(Result,' ',width)
      end
   end

feature {NONE}
------------------------------------------------------------make_fract_part
   make_fract_part : STRING is
   -- this function entered as replacing of simple expression
   -- !!f_f.make(floor((fract_part*10^(prec))+0.5))
   -- which fails in case of large precision
   -- now we obtain fraction part with pieces of decimal_digit_count  digits
      local
         i  : INTEGER
         fp, fff : REAL
         sd : SYSTEM_DEPENDENCE
      do
         -- make fractional part
         if prec = 0 then
            Result := ""
         else
            from
               !!sd
               i := prec
               fp := fract_part
               Result := ""
            until
               i <= sd.decimal_digit_count - 1
            loop
               fff := fp*10^sd.decimal_digit_count
               x_append(Result,produce_piece(floor(fff),sd.decimal_digit_count))
               fp := fff - floor(fff)
               i := i - sd.decimal_digit_count
            end
            x_append(Result,produce_piece(floor((fp*10^i)+0.5),i))
            if zero_strip then
               Result := strip_zeros(Result)
            end
            if not Result.is_equal("") then
            Result.precede('.')
            end
         end
      end

   produce_piece ( i : INTEGER; p : INTEGER) : STRING is
      local
         f_f : UNSIGNED_DECIMAL_INTEGER_FORMATTER
      do
         !!f_f.make(i)
         f_f.set_precision(p)
         f_f.set_align_left
         f_f.set_align_char('0')
         Result := f_f.output
      end

end -- class FIXED_POINT_FORMATTER
